@@ -17,6 +17,7 @@ deserialization of incoming requests."""
 
 import json
 import jsonpatch
+import jsonschema
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
@@ -28,7 +29,7 @@ from glare.api.v1 import api_versioning
 from glare.common import exception as exc
 from glare.common import wsgi
 from glare import engine
-from glare.i18n import _, _LI
+from glare.i18n import _
 
 LOG = logging.getLogger(__name__)
 
@@ -47,16 +48,52 @@ CONF.register_opts(list_configs)
 
 supported_versions = api_versioning.VersionedResource.supported_versions
 
+QUOTA_SCHEMA = {
+    'type': 'object',
+    'properties': {
+        'quota_name': {
+            u'maxLength': 255,
+            u'minLength': 1,
+            u'pattern': u'^[^:]*:?[^:]*$',  # can have only 1 or 0 ':'
+            u'type': u'string'},
+        'quota_value': {'type': 'integer', u'minimum': -1},
+    },
+    'required': ['quota_name', 'quota_value']
+}
+
+QUOTA_INPUT_SCHEMA = {
+    "$schema": "http://json-schema.org/draft-04/schema#",
+    "items": {
+        "properties": {
+            "project_id": {
+                u'maxLength': 255,
+                u'minLength': 1,
+                "type": "string"
+            },
+            "project_quotas": {
+                "items": QUOTA_SCHEMA,
+                "type": "array"
+            }
+        },
+        "type": "object",
+        "required": ["project_id", "project_quotas"]
+    },
+    "type": "array"
+}
+
 
 class RequestDeserializer(api_versioning.VersionedResource,
                           wsgi.JSONRequestDeserializer):
-    """Glare deserializer for incoming webop Requests.
-    Deserializer converts incoming request into bunch of python primitives.
-    So other components doesn't work with requests at all. Deserializer also
-    executes primary API validation without any knowledge about Artifact
-    structure.
+    """Glare deserializer for incoming webob requests.
+
+    Deserializer checks and converts incoming request into a bunch of Glare
+    primitives. So other service components don't work with requests at all.
+    Deserializer also performs primary API validation without any knowledge
+    about concrete artifact type structure.
     """
 
+    ALLOWED_LOCATION_TYPES = ('external', 'internal')
+
     @staticmethod
     def _get_content_type(req, expected=None):
         """Determine content type of the request body."""
@@ -73,7 +110,25 @@ class RequestDeserializer(api_versioning.VersionedResource,
 
         return content_type
 
+    @staticmethod
+    def _get_content_length(req):
+        """Determine content length of the request body."""
+        if req.content_length is None:
+            return
+
+        try:
+            content_length = int(req.content_length)
+            if content_length < 0:
+                raise ValueError
+        except ValueError:
+            msg = _("Content-Length must be a non negative integer.")
+            LOG.error(msg)
+            raise exc.BadRequest(msg)
+
+        return content_length
+
     def _get_request_body(self, req):
+        """Get request json body and convert it to python structures."""
         return self.from_json(req.body)
 
     @supported_versions(min_ver='1.0')
@@ -120,7 +175,7 @@ class RequestDeserializer(api_versioning.VersionedResource,
 
         # step 4 - parse filter parameters
         filters = []
-        for fname, fval in six.iteritems(params):
+        for fname, fval in params.items():
             if fname == 'version' and fval == 'latest':
                 query_params['latest'] = True
             else:
@@ -137,49 +192,68 @@ class RequestDeserializer(api_versioning.VersionedResource,
         patch = jsonpatch.JsonPatch(body)
         try:
             # Initially patch object doesn't validate input. It's only checked
-            # we call get operation on each method
+            # when we call get operation on each method
             tuple(map(patch._get_operation, patch.patch))
-        except (jsonpatch.InvalidJsonPatch, TypeError):
+        except (jsonpatch.InvalidJsonPatch, TypeError, AttributeError,
+                jsonpatch.JsonPointerException):
             msg = _("Json Patch body is malformed")
             raise exc.BadRequest(msg)
-        for patch_item in body:
-            if patch_item['path'] == '/tags':
-                msg = _("Cannot modify artifact tags with PATCH "
-                        "request. Use special Tag API for that.")
-                raise exc.BadRequest(msg)
         return {'patch': patch}
 
-    def _deserialize_blob(self, req):
+    @supported_versions(min_ver='1.0')
+    def upload_blob(self, req):
         content_type = self._get_content_type(req)
+        content_length = self._get_content_length(req)
         if content_type == ('application/vnd+openstack.glare-custom-location'
                             '+json'):
             data = self._get_request_body(req)
             if 'url' not in data:
                 msg = _("url is required when specifying external location. "
-                        "Cannot find url in body: %s") % str(data)
+                        "Cannot find 'url' in request body: %s") % str(data)
                 raise exc.BadRequest(msg)
+            location_type = data.get('location_type', 'external')
+            if location_type not in self.ALLOWED_LOCATION_TYPES:
+                msg = (_("Incorrect location type '%(location_type)s'. It "
+                         "must be one of the following %(allowed)s") %
+                       {'location_type': location_type,
+                        'allowed': ', '.join(self.ALLOWED_LOCATION_TYPES)})
+                raise exc.BadRequest(msg)
+            if location_type == 'external':
+                url = data.get('url')
+                if not url.startswith('http'):
+                    msg = _("Url '%s' doesn't have http(s) scheme") % url
+                    raise exc.BadRequest(msg)
+                if 'md5' not in data:
+                    msg = _("Incorrect blob metadata. MD5 must be specified "
+                            "for external location in artifact blob.")
+                    raise exc.BadRequest(msg)
         else:
             data = req.body_file
-        return {'data': data, 'content_type': content_type}
 
-    @supported_versions(min_ver='1.0')
-    def upload_blob(self, req):
-        return self._deserialize_blob(req)
+        if self.is_valid_encoding(req) and self.is_valid_method(req):
+            req.is_body_readable = True
 
-    @supported_versions(min_ver='1.0')
-    def upload_blob_dict(self, req):
-        return self._deserialize_blob(req)
+        return {'data': data,
+                'content_type': content_type,
+                'content_length': content_length}
 
-    @supported_versions(min_ver='1.0')
-    def set_tags(self, req):
+    @supported_versions(min_ver='1.1')
+    def set_quotas(self, req):
         self._get_content_type(req, expected=['application/json'])
         body = self._get_request_body(req)
+        try:
+            jsonschema.validate(body, QUOTA_INPUT_SCHEMA)
+        except jsonschema.exceptions.ValidationError as e:
+            raise exc.BadRequest(e)
+        values = {}
+        for item in body:
+            project_id = item['project_id']
+            values[project_id] = {}
+            for quota in item['project_quotas']:
+                values[project_id][quota['quota_name']] = quota['quota_value']
+        return {'values': values}
 
-        if 'tags' not in body:
-            msg = _("Tag list must be in the body of request.")
-            raise exc.BadRequest(msg)
-
-        return {'tag_list': body['tags']}
+    # TODO(mfedosin) add pagination to list of quotas
 
 
 def log_request_progress(f):
@@ -189,20 +263,20 @@ def log_request_progress(f):
                   {'request_id': req.context.request_id,
                    'api_method': f.__name__})
         result = f(self, req, *args, **kwargs)
-        LOG.info(_LI(
+        LOG.info(
             "Request %(request_id)s for artifact %(api_method)s "
-            "successfully executed."), {'request_id': req.context.request_id,
-                                        'api_method': f.__name__})
+            "successfully executed.", {'request_id': req.context.request_id,
+                                       'api_method': f.__name__})
         return result
     return log_decorator
 
 
 class ArtifactsController(api_versioning.VersionedResource):
     """API controller for Glare Artifacts.
+
     Artifact Controller prepares incoming data for Glare Engine and redirects
-    data to appropriate engine method (so only controller is working with
-    Engine. Once the data returned from Engine Controller returns data
-    in appropriate format for Response Serializer.
+    data to the appropriate engine method. Once the response data is returned
+    from the engine Controller passes it next to Response Serializer.
     """
 
     def __init__(self):
@@ -211,13 +285,23 @@ class ArtifactsController(api_versioning.VersionedResource):
     @supported_versions(min_ver='1.0')
     @log_request_progress
     def list_type_schemas(self, req):
-        type_schemas = self.engine.list_type_schemas(req.context)
-        return type_schemas
+        """List of detailed descriptions of enabled artifact types.
+
+        :param req: user request
+        :return: list of json-schemas of all enabled artifact types.
+        """
+        return self.engine.show_type_schemas(req.context)
 
     @supported_versions(min_ver='1.0')
     @log_request_progress
     def show_type_schema(self, req, type_name):
-        type_schema = self.engine.show_type_schema(req.context, type_name)
+        """Get detailed artifact type description.
+
+        :param req: user request
+        :param type_name: artifact type name
+        :return: json-schema representation of artifact type
+        """
+        type_schema = self.engine.show_type_schemas(req.context, type_name)
         return {type_name: type_schema}
 
     @supported_versions(min_ver='1.0')
@@ -225,11 +309,21 @@ class ArtifactsController(api_versioning.VersionedResource):
     def create(self, req, type_name, values):
         """Create artifact record in Glare.
 
-        :param req: User request
-        :param type_name: Artifact type name
-        :param values: dict with artifact fields {field_name: field_value}
-        :return definition of created artifact
+        :param req: user request
+        :param type_name: artifact type name
+        :param values: dict with artifact fields
+        :return: definition of created artifact
         """
+        if req.context.tenant is None or req.context.read_only:
+            msg = _("It's forbidden to anonymous users to create artifacts.")
+            raise exc.Forbidden(msg)
+        if not values.get('name'):
+            msg = _("Name must be specified at creation.")
+            raise exc.BadRequest(msg)
+        for field in ('visibility', 'status'):
+            if field in values:
+                msg = _("%s is not allowed in a request at creation.") % field
+                raise exc.BadRequest(msg)
         return self.engine.create(req.context, type_name, values)
 
     @supported_versions(min_ver='1.0')
@@ -241,14 +335,14 @@ class ArtifactsController(api_versioning.VersionedResource):
         :param type_name: Artifact type name
         :param artifact_id: id of artifact to update
         :param patch: json patch with artifact changes
-        :return definition of updated artifact
+        :return: definition of updated artifact
         """
-        return self.engine.update(req.context, type_name, artifact_id, patch)
+        return self.engine.save(req.context, type_name, artifact_id, patch)
 
     @supported_versions(min_ver='1.0')
     @log_request_progress
     def delete(self, req, type_name, artifact_id):
-        """Delete artifact from Glare
+        """Delete artifact from Glare.
 
         :param req: User request
         :param type_name: Artifact type name
@@ -259,20 +353,20 @@ class ArtifactsController(api_versioning.VersionedResource):
     @supported_versions(min_ver='1.0')
     @log_request_progress
     def show(self, req, type_name, artifact_id):
-        """Show detailed artifact info
+        """Show detailed artifact info.
 
         :param req: User request
         :param type_name: Artifact type name
         :param artifact_id: id of artifact to show
         :return: definition of requested artifact
         """
-        return self.engine.get(req.context, type_name, artifact_id)
+        return self.engine.show(req.context, type_name, artifact_id)
 
     @supported_versions(min_ver='1.0')
     @log_request_progress
-    def list(self, req, type_name, filters, marker=None, limit=None,
+    def list(self, req, type_name, filters=None, marker=None, limit=None,
              sort=None, latest=False):
-        """List available artifacts
+        """List available artifacts.
 
         :param req: User request
         :param type_name: Artifact type name
@@ -284,7 +378,7 @@ class ArtifactsController(api_versioning.VersionedResource):
         :param sort: sorting options
         :param latest: flag that indicates, that only artifacts with highest
         versions should be returned in output
-        :return: list of artifacts
+        :return: list of requested artifact definitions
         """
         artifacts = self.engine.list(req.context, type_name, filters, marker,
                                      limit, sort, latest)
@@ -294,127 +388,117 @@ class ArtifactsController(api_versioning.VersionedResource):
             result['next_marker'] = artifacts[-1]['id']
         return result
 
+    @staticmethod
+    def _parse_blob_path(blob_path):
+        field_name, _sep, blob_key = blob_path.partition('/')
+        if not blob_key:
+            blob_key = None
+        return field_name, blob_key
+
     @supported_versions(min_ver='1.0')
     @log_request_progress
-    def upload_blob(self, req, type_name, artifact_id, field_name, data,
-                    content_type):
-        """Upload blob into Glare repo
+    def upload_blob(self, req, type_name, artifact_id, blob_path, data,
+                    content_type, content_length=None):
+        """Upload blob into Glare repo.
 
         :param req: User request
         :param type_name: Artifact type name
-        :param artifact_id: id of Artifact to reactivate
-        :param field_name: name of blob field in artifact
-        :param data: Artifact payload
+        :param artifact_id: id of artifact where to perform upload
+        :param blob_path: path to artifact blob
+        :param data: blob payload
         :param content_type: data content-type
+        :param content_length: amount of data user wants to upload
+        :return: definition of requested artifact with uploaded blob
         """
+        field_name, blob_key = self._parse_blob_path(blob_path)
         if content_type == ('application/vnd+openstack.glare-custom-location'
                             '+json'):
             url = data.pop('url')
             return self.engine.add_blob_location(
-                req.context, type_name, artifact_id, field_name, url, data)
+                req.context, type_name, artifact_id, field_name, url, data,
+                blob_key)
         else:
-            return self.engine.upload_blob(req.context, type_name, artifact_id,
-                                           field_name, data, content_type)
+            return self.engine.upload_blob(
+                req.context, type_name, artifact_id, field_name, data,
+                content_type, content_length, blob_key)
 
     @supported_versions(min_ver='1.0')
     @log_request_progress
-    def upload_blob_dict(self, req, type_name, artifact_id, field_name, data,
-                         blob_key, content_type):
-        """Upload blob into Glare repo
+    def download_blob(self, req, type_name, artifact_id, blob_path):
+        """Download blob data from Artifact.
 
         :param req: User request
-        :param type_name: Artifact type name
-        :param artifact_id: id of Artifact to reactivate
-        :param field_name: name of blob field in artifact
-        :param data: Artifact payload
-        :param content_type: data content-type
-        :param blob_key: blob key in dict
+        :param type_name: artifact type name
+        :param artifact_id: id of artifact from where to perform download
+        :param blob_path: path to artifact blob
+        :return: requested blob data
         """
-        if content_type == ('application/vnd+openstack.glare-custom-location'
-                            '+json'):
-            url = data.pop('url')
-            return self.engine.add_blob_dict_location(
-                req.context, type_name, artifact_id,
-                field_name, blob_key, url, data)
-        else:
-            return self.engine.upload_blob_dict(
-                req.context, type_name, artifact_id,
-                field_name, blob_key, data, content_type)
-
-    @supported_versions(min_ver='1.0')
-    @log_request_progress
-    def download_blob(self, req, type_name, artifact_id, field_name):
-        """Download blob data from Artifact
-
-        :param req: User request
-        :param type_name: Artifact type name
-        :param artifact_id: id of Artifact to reactivate
-        :param field_name: name of blob field in artifact
-        :return: iterator that returns blob data
-        """
-        data, meta = self.engine.download_blob(req.context, type_name,
-                                               artifact_id, field_name)
-        result = {'data': data, 'meta': meta}
-        return result
-
-    @supported_versions(min_ver='1.0')
-    @log_request_progress
-    def download_blob_dict(self, req, type_name, artifact_id,
-                           field_name, blob_key):
-        """Download blob data from Artifact
-
-        :param req: User request
-        :param type_name: Artifact type name
-        :param artifact_id: id of Artifact to reactivate
-        :param field_name: name of blob field in artifact
-        :param blob_key: name of Dict of blobs (optional)
-        :return: iterator that returns blob data
-        """
-        data, meta = self.engine.download_blob_dict(
+        field_name, blob_key = self._parse_blob_path(blob_path)
+        data, meta = self.engine.download_blob(
             req.context, type_name, artifact_id, field_name, blob_key)
         result = {'data': data, 'meta': meta}
         return result
 
-    @staticmethod
-    def _tag_body_resp(af):
-        return {'tags': af['tags']}
-
-    @supported_versions(min_ver='1.0')
+    @supported_versions(min_ver='1.1')
     @log_request_progress
-    def get_tags(self, req, type_name, artifact_id):
-        return self._tag_body_resp(self.engine.get(
-            req.context, type_name, artifact_id))
+    def delete_external_blob(self, req, type_name, artifact_id, blob_path):
+        """Delete blob with external location from Glare repo.
 
-    @supported_versions(min_ver='1.0')
-    @log_request_progress
-    def set_tags(self, req, type_name, artifact_id, tag_list):
-        patch = [{'op': 'replace', 'path': '/tags', 'value': tag_list}]
-        patch = jsonpatch.JsonPatch(patch)
-        return self._tag_body_resp(self.engine.update(
-            req.context, type_name, artifact_id, patch))
+        :param req: User request
+        :param type_name: Artifact type name
+        :param artifact_id: id of artifact with the blob to delete
+        :param blob_path: path to artifact blob
+        """
+        field_name, blob_key = self._parse_blob_path(blob_path)
+        return self.engine.delete_external_blob(
+            req.context, type_name, artifact_id, field_name, blob_key)
 
-    @supported_versions(min_ver='1.0')
+    @supported_versions(min_ver='1.1')
     @log_request_progress
-    def delete_tags(self, req, type_name, artifact_id):
-        patch = [{'op': 'replace', 'path': '/tags', 'value': []}]
-        patch = jsonpatch.JsonPatch(patch)
-        self.engine.update(req.context, type_name, artifact_id, patch)
+    def set_quotas(self, req, values):
+        """Set quota records in Glare.
+
+        :param req: user request
+        :param values: list with quota values to set
+        """
+        self.engine.set_quotas(req.context, values)
+
+    @supported_versions(min_ver='1.1')
+    @log_request_progress
+    def list_all_quotas(self, req):
+        """Get detailed info about all available quotas.
+
+        :param req: user request
+        :return: definition of requested quotas for the project
+        """
+        return self.engine.list_all_quotas(req.context)
+
+    @supported_versions(min_ver='1.1')
+    @log_request_progress
+    def list_project_quotas(self, req, project_id=None):
+        """Get detailed info about project quotas.
+
+        :param req: user request
+        :param project_id: id of the project for which to show quotas
+        :return: definition of requested quotas for the project
+        """
+        return self.engine.list_project_quotas(req.context, project_id)
 
 
 class ResponseSerializer(api_versioning.VersionedResource,
                          wsgi.JSONResponseSerializer):
-    """Glare Response Serializer converts data received from Glare Engine
-    (it consists from plain data types - dict, int, string, file descriptors,
-    etc) to WSGI Requests. It also specifies proper response status and
-    content type as specified by API design.
+    """Glare serializer for outgoing responses.
+
+    Converts data received from the engine to WSGI responses. It also
+    specifies proper response status and content type as declared in the API.
     """
 
     @staticmethod
     def _prepare_json_response(response, result,
                                content_type='application/json'):
         body = json.dumps(result, ensure_ascii=False)
-        response.unicode_body = six.text_type(body)
-        response.content_type = content_type
+        response.text = six.text_type(body)
+        response.content_type = content_type + '; charset=UTF-8'
 
     def list_type_schemas(self, response, type_schemas):
         self._prepare_json_response(response,
@@ -449,7 +533,7 @@ class ResponseSerializer(api_versioning.VersionedResource,
         params.pop('marker', None)
 
         encode_params = {}
-        for key, value in six.iteritems(params):
+        for key, value in params.items():
             encode_params[key] = encodeutils.safe_encode(value)
         query = urlparse.urlencode(encode_params)
 
@@ -465,9 +549,8 @@ class ResponseSerializer(api_versioning.VersionedResource,
             params['marker'] = af_list['next_marker']
             next_query = urlparse.urlencode(params)
             body['next'] = '/artifacts/%s?%s' % (type_name, next_query)
-        response.unicode_body = six.text_type(json.dumps(body,
-                                                         ensure_ascii=False))
-        response.content_type = 'application/json'
+
+        self._prepare_json_response(response, body)
 
     @supported_versions(min_ver='1.0')
     def delete(self, response, result):
@@ -480,11 +563,11 @@ class ResponseSerializer(api_versioning.VersionedResource,
     @staticmethod
     def _serialize_blob(response, result):
         data, meta = result['data'], result['meta']
+        response.app_iter = iter(data)
         response.headers['Content-Type'] = meta['content_type']
         response.headers['Content-MD5'] = meta['md5']
         response.headers['X-Openstack-Glare-Content-SHA1'] = meta['sha1']
         response.headers['X-Openstack-Glare-Content-SHA256'] = meta['sha256']
-        response.app_iter = iter(data)
         response.content_length = str(meta['size'])
 
     @staticmethod
@@ -506,21 +589,39 @@ class ResponseSerializer(api_versioning.VersionedResource,
         else:
             self._serialize_blob(response, result)
 
-    @supported_versions(min_ver='1.0')
-    def download_blob_dict(self, response, result):
-        external = result['meta']['external']
-        if external:
-            self._serialize_location(response, result)
-        else:
-            self._serialize_blob(response, result)
+    @supported_versions(min_ver='1.1')
+    def delete_external_blob(self, response, result):
+        self._prepare_json_response(response, result)
 
-    @supported_versions(min_ver='1.0')
-    def delete_tags(self, response, result):
-        response.status_int = http_client.NO_CONTENT
+    @staticmethod
+    def _serialize_quota(quotas):
+        res = []
+        for project_id, project_quotas in quotas.items():
+            quota_list = []
+            for quota_name, quota_value in project_quotas.items():
+                quota_list.append({
+                    'quota_name': quota_name,
+                    'quota_value': quota_value,
+                })
+            res.append({
+                'project_id': project_id,
+                'project_quotas': quota_list
+            })
+        return res
+
+    @supported_versions(min_ver='1.1')
+    def list_all_quotas(self, response, quotas):
+        quotas['quotas'] = self._serialize_quota(quotas['quotas'])
+        self._prepare_json_response(response, quotas)
+
+    @supported_versions(min_ver='1.1')
+    def list_project_quotas(self, response, quotas):
+        quotas = self._serialize_quota(quotas)
+        self._prepare_json_response(response, quotas)
 
 
 def create_resource():
-    """Artifact resource factory method"""
+    """Artifact resource factory method."""
     deserializer = RequestDeserializer()
     serializer = ResponseSerializer()
     controller = ArtifactsController()
