@@ -15,19 +15,12 @@
 
 import jsonschema
 
-from glare.common import utils
 from glare.tests.functional import base
 
 fixture_base_props = {
     u'activated_at': {
         u'description': u'Datetime when artifact has became active.',
-        u'filter_ops': [u'eq',
-                        u'neq',
-                        u'in',
-                        u'gt',
-                        u'gte',
-                        u'lt',
-                        u'lte'],
+        u'filter_ops': [u'lt', u'gt'],
         u'format': u'date-time',
         u'glareType': u'DateTime',
         u'readOnly': True,
@@ -37,13 +30,7 @@ fixture_base_props = {
                   u'null']},
     u'created_at': {
         u'description': u'Datetime when artifact has been created.',
-        u'filter_ops': [u'eq',
-                        u'neq',
-                        u'in',
-                        u'gt',
-                        u'gte',
-                        u'lt',
-                        u'lte'],
+        u'filter_ops': [u'lt', u'gt'],
         u'format': u'date-time',
         u'glareType': u'DateTime',
         u'readOnly': True,
@@ -51,38 +38,13 @@ fixture_base_props = {
         u'type': u'string'},
     u'description': {u'default': u'',
                      u'description': u'Artifact description.',
-                     u'filter_ops': [u'eq',
-                                     u'neq',
-                                     u'in'],
+                     u'filter_ops': [],
                      u'glareType': u'String',
                      u'maxLength': 4096,
                      u'mutable': True,
                      u'required_on_activate': False,
                      u'type': [u'string',
                                u'null']},
-    u'icon': {u'additionalProperties': False,
-              u'description': u'Artifact icon.',
-              u'filter_ops': [],
-              u'glareType': u'Blob',
-              u'properties': {u'md5': {u'type': [u'string', u'null']},
-                              u'sha1': {u'type': [u'string', u'null']},
-                              u'sha256': {u'type': [u'string', u'null']},
-                              u'content_type': {u'type': u'string'},
-                              u'external': {u'type': u'boolean'},
-                              u'size': {u'type': [u'number',
-                                                  u'null']},
-                              u'status': {u'enum': [u'saving',
-                                                    u'active',
-                                                    u'pending_delete'],
-                                          u'type': u'string'}},
-              u'required': [u'size',
-                            u'md5', u'sha1', u'sha256',
-                            u'external',
-                            u'status',
-                            u'content_type'],
-              u'required_on_activate': False,
-              u'type': [u'object',
-                        u'null']},
     u'id': {u'description': u'Artifact UUID.',
             u'filter_ops': [u'eq',
                             u'neq',
@@ -94,30 +56,13 @@ fixture_base_props = {
             u'readOnly': True,
             u'sortable': True,
             u'type': u'string'},
-    u'license': {u'description': u'Artifact license type.',
-                 u'filter_ops': [u'eq',
-                                 u'neq',
-                                 u'in'],
-                 u'glareType': u'String',
-                 u'maxLength': 255,
-                 u'required_on_activate': False,
-                 u'type': [u'string',
-                           u'null']},
-    u'license_url': {u'description': u'URL to artifact license.',
-                     u'filter_ops': [u'eq',
-                                     u'neq',
-                                     u'in'],
-                     u'glareType': u'String',
-                     u'maxLength': 255,
-                     u'required_on_activate': False,
-                     u'type': [u'string',
-                               u'null']},
-    u'metadata': {u'additionalProperties': {u'type': u'string'},
+    u'metadata': {u'additionalProperties': {u'maxLength': 255,
+                                            u'minLength': 1,
+                                            u'type': u'string'},
                   u'default': {},
                   u'description': u'Key-value dict with useful information '
                                   u'about an artifact.',
-                  u'filter_ops': [u'eq',
-                                  u'neq'],
+                  u'filter_ops': [u'eq', u'neq', u'in'],
                   u'glareType': u'StringDict',
                   u'maxProperties': 255,
                   u'required_on_activate': False,
@@ -129,6 +74,7 @@ fixture_base_props = {
                               u'in'],
               u'glareType': u'String',
               u'maxLength': 255,
+              u'minLength': 1,
               u'required_on_activate': False,
               u'sortable': True,
               u'type': u'string'},
@@ -142,33 +88,6 @@ fixture_base_props = {
                u'required_on_activate': False,
                u'sortable': True,
                u'type': u'string'},
-    u'provided_by': {u'additionalProperties': False,
-                     u'description': u'Info about artifact authors.',
-                     u'filter_ops': [u'eq',
-                                     u'neq',
-                                     u'in'],
-                     u'glareType': u'StringDict',
-                     u'maxProperties': 255,
-                     u'properties': {u'company': {u'type': u'string'},
-                                     u'href': {u'type': u'string'},
-                                     u'name': {u'type': u'string'}},
-                     u'required_on_activate': False,
-                     u'type': [u'object',
-                               u'null']},
-    u'release': {u'default': [],
-                 u'description': u'Target Openstack release for artifact. It '
-                                 u'is usually the same when artifact was '
-                                 u'uploaded.',
-                 u'filter_ops': [u'eq',
-                                 u'neq',
-                                 u'in'],
-                 u'glareType': u'StringList',
-                 u'items': {u'type': u'string'},
-                 u'maxItems': 255,
-                 u'required_on_activate': False,
-                 u'type': [u'array',
-                           u'null'],
-                 u'unique': True},
     u'status': {u'default': u'drafted',
                 u'description': u'Artifact status.',
                 u'enum': [u'drafted',
@@ -179,43 +98,28 @@ fixture_base_props = {
                                 u'neq',
                                 u'in'],
                 u'glareType': u'String',
+                u'mutable': True,
                 u'sortable': True,
                 u'type': u'string'},
-    u'supported_by': {u'additionalProperties': {u'type': u'string'},
-                      u'description': u'Info about persons who responsible '
-                                      u'for artifact support',
-                      u'filter_ops': [u'eq',
-                                      u'neq',
-                                      u'in'],
-                      u'glareType': u'StringDict',
-                      u'maxProperties': 255,
-                      u'required': [u'name'],
-                      u'required_on_activate': False,
-                      u'type': [u'object',
-                                u'null']},
     u'tags': {u'default': [],
               u'description': u'List of tags added to Artifact.',
-              u'filter_ops': [u'eq',
-                              u'neq',
-                              u'in'],
+              u'filter_ops': [],
               u'glareType': u'StringList',
-              u'items': {u'type': u'string'},
+              u'items': {u'maxLength': 255,
+                         u'minLength': 1,
+                         u'pattern': u'^[^,/]+$',
+                         u'type': u'string'},
               u'maxItems': 255,
               u'mutable': True,
               u'required_on_activate': False,
-              u'type': [u'array',
-                        u'null']},
+              u'type': [u'array', u'null'],
+              u'uniqueItems': True},
     u'updated_at': {
         u'description': u'Datetime when artifact has been updated last time.',
-        u'filter_ops': [u'eq',
-                        u'neq',
-                        u'in',
-                        u'gt',
-                        u'gte',
-                        u'lt',
-                        u'lte'],
+        u'filter_ops': [u'lt', u'gt'],
         u'format': u'date-time',
         u'glareType': u'DateTime',
+        u'mutable': True,
         u'readOnly': True,
         u'sortable': True,
         u'type': u'string'},
@@ -239,9 +143,10 @@ fixture_base_props = {
                     u'description': u'Artifact visibility that defines if '
                                     u'artifact can be available to other '
                                     u'users.',
-                    u'filter_ops': [u'eq'],
+                    u'enum': [u'private', u'public'],
+                    u'filter_ops': [u'eq', u'neq', u'in'],
                     u'glareType': u'String',
-                    u'maxLength': 255,
+                    u'mutable': True,
                     u'sortable': True,
                     u'type': u'string'}
 }
@@ -275,8 +180,7 @@ fixtures = {
                           u'status': {
                               u'enum': [
                                   u'saving',
-                                  u'active',
-                                  u'pending_delete'],
+                                  u'active'],
                               u'type': u'string'}},
                       u'required': [u'size',
                                     u'md5', u'sha1', u'sha256',
@@ -290,24 +194,22 @@ fixtures = {
                        u'filter_ops': [u'eq'],
                        u'glareType': u'Boolean',
                        u'required_on_activate': False,
-                       u'type': [u'string',
+                       u'type': [u'boolean',
                                  u'null']},
             u'bool2': {u'default': False,
                        u'filter_ops': [u'eq'],
                        u'glareType': u'Boolean',
                        u'required_on_activate': False,
-                       u'type': [u'string',
+                       u'type': [u'boolean',
                                  u'null']},
             u'link1': {u'filter_ops': [u'eq',
-                                       u'neq',
-                                       u'in'],
+                                       u'neq'],
                        u'glareType': u'Link',
                        u'required_on_activate': False,
                        u'type': [u'string',
                                  u'null']},
             u'link2': {u'filter_ops': [u'eq',
-                                       u'neq',
-                                       u'in'],
+                                       u'neq'],
                        u'glareType': u'Link',
                        u'required_on_activate': False,
                        u'type': [u'string',
@@ -330,8 +232,7 @@ fixtures = {
                         u'status': {
                             u'enum': [
                                 u'saving',
-                                u'active',
-                                u'pending_delete'],
+                                u'active'],
                             u'type': u'string'}},
                     u'required': [u'size',
                                   u'md5', u'sha1', u'sha256',
@@ -349,19 +250,35 @@ fixtures = {
                           u'null']},
             u'dict_of_int': {
                 u'additionalProperties': {
-                    u'type': u'string'},
+                    u'type': u'integer'},
                 u'default': {},
-                u'filter_ops': [u'eq'],
+                u'filter_ops': [u'eq', u'in'],
                 u'glareType': u'IntegerDict',
                 u'maxProperties': 255,
                 u'required_on_activate': False,
                 u'type': [u'object',
                           u'null']},
+            u'dict_of_links': {u'additionalProperties': {u'type': u'string'},
+                               u'default': {},
+                               u'filter_ops': [u'eq'],
+                               u'glareType': u'LinkDict',
+                               u'maxProperties': 255,
+                               u'mutable': True,
+                               u'required_on_activate': False,
+                               u'type': [u'object', u'null']},
+            u'list_of_links': {u'default': [],
+                               u'filter_ops': [u'eq'],
+                               u'glareType': u'LinkList',
+                               u'items': {u'type': u'string'},
+                               u'maxItems': 255,
+                               u'mutable': True,
+                               u'required_on_activate': False,
+                               u'type': [u'array', u'null']},
             u'dict_of_str': {
-                u'additionalProperties': {
-                    u'type': u'string'},
+                u'additionalProperties': {u'maxLength': 255,
+                                          u'type': u'string'},
                 u'default': {},
-                u'filter_ops': [u'eq'],
+                u'filter_ops': [u'eq', u'in'],
                 u'glareType': u'StringDict',
                 u'maxProperties': 255,
                 u'required_on_activate': False,
@@ -369,20 +286,21 @@ fixtures = {
                           u'null']},
             u'dict_validators': {
                 u'additionalProperties': False,
-                u'filter_ops': [u'eq',
-                                u'neq',
-                                u'in'],
+                u'filter_ops': [],
                 u'glareType': u'StringDict',
                 u'maxProperties': 3,
-                u'properties': {
-                    u'abc': {u'type': [u'string',
-                                       u'null']},
-                    u'def': {u'type': [u'string',
-                                       u'null']},
-                    u'ghi': {u'type': [u'string',
-                                       u'null']},
-                    u'jkl': {u'type': [u'string',
-                                       u'null']}},
+                u'properties': {u'abc': {u'maxLength': 255,
+                                         u'type': [u'string',
+                                                   u'null']},
+                                u'def': {u'maxLength': 255,
+                                         u'type': [u'string',
+                                                   u'null']},
+                                u'ghi': {u'maxLength': 255,
+                                         u'type': [u'string',
+                                                   u'null']},
+                                u'jkl': {u'maxLength': 255,
+                                         u'type': [u'string',
+                                                   u'null']}},
                 u'required_on_activate': False,
                 u'type': [u'object',
                           u'null']},
@@ -448,36 +366,33 @@ fixtures = {
                                 u'type': [u'integer',
                                           u'null']},
             u'list_of_int': {u'default': [],
-                             u'filter_ops': [u'eq'],
+                             u'filter_ops': [u'eq', u'in'],
                              u'glareType': u'IntegerList',
                              u'items': {
-                                 u'type': u'string'},
+                                 u'type': u'integer'},
                              u'maxItems': 255,
                              u'required_on_activate': False,
                              u'type': [u'array',
                                        u'null']},
             u'list_of_str': {u'default': [],
-                             u'filter_ops': [u'eq'],
+                             u'filter_ops': [u'eq', u'in'],
                              u'glareType': u'StringList',
-                             u'items': {
-                                 u'type': u'string'},
+                             u'items': {u'maxLength': 255,
+                                        u'type': u'string'},
                              u'maxItems': 255,
                              u'required_on_activate': False,
                              u'type': [u'array',
                                        u'null']},
             u'list_validators': {u'default': [],
-                                 u'filter_ops': [
-                                     u'eq',
-                                     u'neq',
-                                     u'in'],
+                                 u'filter_ops': [],
                                  u'glareType': u'StringList',
-                                 u'items': {
-                                     u'type': u'string'},
+                                 u'items': {u'maxLength': 255,
+                                            u'type': u'string'},
                                  u'maxItems': 3,
                                  u'required_on_activate': False,
                                  u'type': [u'array',
                                            u'null'],
-                                 u'unique': True},
+                                 u'uniqueItems': True},
             u'small_blob': {u'additionalProperties': False,
                             u'filter_ops': [],
                             u'glareType': u'Blob',
@@ -497,8 +412,7 @@ fixtures = {
                                 u'status': {
                                     u'enum': [
                                         u'saving',
-                                        u'active',
-                                        u'pending_delete'],
+                                        u'active'],
                                     u'type': u'string'}},
                             u'required': [u'size',
                                           u'md5', u'sha1', u'sha256',
@@ -510,11 +424,7 @@ fixtures = {
                                       u'null']},
             u'str1': {u'filter_ops': [u'eq',
                                       u'neq',
-                                      u'in',
-                                      u'gt',
-                                      u'gte',
-                                      u'lt',
-                                      u'lte'],
+                                      u'in'],
                       u'glareType': u'String',
                       u'maxLength': 255,
                       u'required_on_activate': False,
@@ -523,25 +433,23 @@ fixtures = {
                                 u'null']},
             u'string_mutable': {u'filter_ops': [u'eq',
                                                 u'neq',
-                                                u'in',
-                                                u'gt',
-                                                u'gte',
-                                                u'lt',
-                                                u'lte'],
+                                                u'in'],
                                 u'glareType': u'String',
                                 u'maxLength': 255,
                                 u'mutable': True,
                                 u'required_on_activate': False,
                                 u'type': [u'string',
                                           u'null']},
+            u'string_regex': {u'filter_ops': [u'eq', u'neq', u'in'],
+                              u'glareType': u'String',
+                              u'maxLength': 255,
+                              u'pattern': u'^([0-9a-fA-F]){8}$',
+                              u'required_on_activate': False,
+                              u'type': [u'string', u'null']},
             u'string_required': {
                 u'filter_ops': [u'eq',
                                 u'neq',
-                                u'in',
-                                u'gt',
-                                u'gte',
-                                u'lt',
-                                u'lte'],
+                                u'in'],
                 u'glareType': u'String',
                 u'maxLength': 255,
                 u'type': [u'string',
@@ -549,15 +457,10 @@ fixtures = {
             u'string_validators': {
                 u'enum': [u'aa',
                           u'bb',
-                          u'ccccccccccc',
-                          None],
+                          u'ccccccccccc'],
                 u'filter_ops': [u'eq',
                                 u'neq',
-                                u'in',
-                                u'gt',
-                                u'gte',
-                                u'lt',
-                                u'lte'],
+                                u'in'],
                 u'glareType': u'String',
                 u'maxLength': 10,
                 u'required_on_activate': False,
@@ -596,8 +499,7 @@ fixtures = {
                     u'size': {u'type': [u'number',
                                         u'null']},
                     u'status': {u'enum': [u'saving',
-                                          u'active',
-                                          u'pending_delete'],
+                                          u'active'],
                                 u'type': u'string'}},
                 u'required': [u'size',
                               u'md5', u'sha1', u'sha256',
@@ -630,7 +532,8 @@ fixtures = {
                                 u'neq',
                                 u'in'],
                 u'glareType': u'StringList',
-                u'items': {u'type': u'string'},
+                u'items': {u'maxLength': 255,
+                           u'type': u'string'},
                 u'maxItems': 255,
                 u'mutable': True,
                 u'type': [u'array',
@@ -643,18 +546,18 @@ fixtures = {
                                 u'neq',
                                 u'in'],
                 u'glareType': u'StringList',
-                u'items': {u'type': u'string'},
+                u'items': {u'maxLength': 255,
+                           u'type': u'string'},
                 u'maxItems': 255,
                 u'type': [u'array',
                           u'null'],
-                u'unique': True},
+                u'uniqueItems': True},
             u'dependencies': {
                 u'default': [],
                 u'description': u'List of package dependencies for '
                                 u'this package.',
                 u'filter_ops': [u'eq',
-                                u'neq',
-                                u'in'],
+                                u'neq'],
                 u'glareType': u'LinkList',
                 u'items': {u'type': u'string'},
                 u'maxItems': 255,
@@ -672,7 +575,8 @@ fixtures = {
                 u'type': [u'string',
                           u'null']},
             u'inherits': {
-                u'additionalProperties': {u'type': u'string'},
+                u'additionalProperties': {u'maxLength': 255,
+                                          u'type': u'string'},
                 u'default': {},
                 u'filter_ops': [u'eq',
                                 u'neq',
@@ -686,7 +590,8 @@ fixtures = {
                                           u'neq',
                                           u'in'],
                           u'glareType': u'StringList',
-                          u'items': {u'type': u'string'},
+                          u'items': {u'maxLength': 255,
+                                     u'type': u'string'},
                           u'maxItems': 255,
                           u'mutable': True,
                           u'type': [u'array',
@@ -704,8 +609,7 @@ fixtures = {
                                 u'size': {u'type': [u'number',
                                                     u'null']},
                                 u'status': {u'enum': [u'saving',
-                                                      u'active',
-                                                      u'pending_delete'],
+                                                      u'active'],
                                             u'type': u'string'}},
                 u'required': [u'size',
                               u'md5', u'sha1', u'sha256',
@@ -720,12 +624,11 @@ fixtures = {
                 u'description': u'Package type.',
                 u'enum': [u'Application',
                           u'Library',
-                          None],
+                          ],
                 u'filter_ops': [u'eq',
                                 u'neq',
                                 u'in'],
                 u'glareType': u'String',
-                u'maxLength': 255,
                 u'type': [u'string',
                           u'null']}
         }),
@@ -749,14 +652,6 @@ fixtures = {
                 u'required_on_activate': False,
                 u'type': [u'string',
                           u'null']},
-            u'cloud_user': {u'description': u'Default cloud user.',
-                            u'filter_ops': [u'eq',
-                                            u'neq',
-                                            u'in'],
-                            u'glareType': u'String',
-                            u'maxLength': 255,
-                            u'required_on_activate': False,
-                            u'type': [u'string', u'null']},
             u'container_format': {u'description': u'Image container format.',
                                   u'enum': [u'ami',
                                             u'ari',
@@ -765,12 +660,11 @@ fixtures = {
                                             u'ovf',
                                             u'ova',
                                             u'docker',
-                                            None],
+                                            ],
                                   u'filter_ops': [u'eq',
                                                   u'neq',
                                                   u'in'],
                                   u'glareType': u'String',
-                                  u'maxLength': 255,
                                   u'type': [u'string',
                                             u'null']},
 
@@ -785,12 +679,11 @@ fixtures = {
                                        u'qcow2',
                                        u'vdi',
                                        u'iso',
-                                       None],
+                                       ],
                              u'filter_ops': [u'eq',
                                              u'neq',
                                              u'in'],
                              u'glareType': u'String',
-                             u'maxLength': 255,
                              u'type': [u'string', u'null']},
             u'image': {u'additionalProperties': False,
                        u'description': u'Image binary.',
@@ -805,8 +698,7 @@ fixtures = {
                            u'size': {u'type': [u'number',
                                                u'null']},
                            u'status': {u'enum': [u'saving',
-                                                 u'active',
-                                                 u'pending_delete'],
+                                                 u'active'],
                                        u'type': u'string'}},
                        u'required': [u'size',
                                      u'md5', u'sha1', u'sha256',
@@ -815,20 +707,6 @@ fixtures = {
                                      u'content_type'],
                        u'required_on_activate': False,
                        u'type': [u'object', u'null']},
-            u'image_indirect_url': {
-                u'description': u'URL where image is available for users by '
-                                u'accepting EULA or some other form. It is '
-                                u'used when it is not possible to upload '
-                                u'image directly to Glare. F.e. some Windows '
-                                u'cloud images requires EULA acceptance '
-                                u'before download.',
-                u'filter_ops': [u'eq',
-                                u'neq',
-                                u'in'],
-                u'glareType': u'String',
-                u'maxLength': 255,
-                u'required_on_activate': False,
-                u'type': [u'string', u'null']},
             u'instance_uuid': {
                 u'description': u'Metadata which can be used to record which '
                                 u'instance this image is associated with. '
@@ -860,7 +738,11 @@ fixtures = {
                 u'description': u'Minimal disk space required to boot image.',
                 u'filter_ops': [u'eq',
                                 u'neq',
-                                u'in'],
+                                u'in',
+                                u'gt',
+                                u'gte',
+                                u'lt',
+                                u'lte'],
                 u'glareType': u'Integer',
                 u'minimum': 0,
                 u'required_on_activate': False,
@@ -869,7 +751,11 @@ fixtures = {
                 u'description': u'Minimal RAM required to boot image.',
                 u'filter_ops': [u'eq',
                                 u'neq',
-                                u'in'],
+                                u'in',
+                                u'gt',
+                                u'gte',
+                                u'lt',
+                                u'lte'],
                 u'glareType': u'Integer',
                 u'minimum': 0,
                 u'required_on_activate': False,
@@ -917,7 +803,8 @@ fixtures = {
         u'name': u'heat_templates',
         u'properties': generate_type_props({
             u'default_envs': {
-                u'additionalProperties': {u'type': u'string'},
+                u'additionalProperties': {u'maxLength': 255,
+                                          u'type': u'string'},
                 u'default': {},
                 u'description': u'Default environments that can '
                                 u'be applied to the template if no '
@@ -937,8 +824,7 @@ fixtures = {
                                 u'that can be used with current '
                                 u'template.',
                 u'filter_ops': [u'eq',
-                                u'neq',
-                                u'in'],
+                                u'neq'],
                 u'glareType': u'LinkDict',
                 u'maxProperties': 255,
                 u'mutable': True,
@@ -957,8 +843,7 @@ fixtures = {
                          u'size': {u'type': [u'number',
                                              u'null']},
                          u'status': {u'enum': [u'saving',
-                                               u'active',
-                                               u'pending_delete'],
+                                               u'active'],
                                      u'type': u'string'}},
                      u'required': [u'size',
                                    u'md5', u'sha1', u'sha256',
@@ -991,8 +876,7 @@ fixtures = {
                     u'size': {u'type': [u'number',
                                         u'null']},
                     u'status': {u'enum': [u'saving',
-                                          u'active',
-                                          u'pending_delete'],
+                                          u'active'],
                                 u'type': u'string'}},
                 u'required': [u'size',
                               u'md5', u'sha1', u'sha256',
@@ -1023,8 +907,7 @@ fixtures = {
                                 u'size': {u'type': [u'number',
                                                     u'null']},
                                 u'status': {u'enum': [u'saving',
-                                                      u'active',
-                                                      u'pending_delete'],
+                                                      u'active'],
                                             u'type': u'string'}},
                 u'required': [u'size',
                               u'md5', u'sha1', u'sha256',
@@ -1061,15 +944,11 @@ class TestSchemas(base.TestArtifact):
         # Get schemas for specific artifact type
         for at in self.enabled_types:
             result = self.get(url='/schemas/%s' % at)
-            self.assertEqual(fixtures[at], result['schemas'][at],
-                             utils.DictDiffer(
-                                 fixtures[at]['properties'],
-                                 result['schemas'][at]['properties']))
+            self.assertEqual(fixtures[at], result['schemas'][at])
 
         # Get list schemas of artifacts
         result = self.get(url='/schemas')
-        self.assertEqual(fixtures, result['schemas'], utils.DictDiffer(
-            fixtures, result['schemas']))
+        self.assertEqual(fixtures, result['schemas'])
 
         # Validation of schemas
         result = self.get(url='/schemas')['schemas']
